"use strict";
// Copyright (C) 1996 Cobasoft GmbH, Karl-W.Geitz. See cs_license.txt!
// COBASOFT BASE JAVASCRIPT UTILITIES
var CS = {
	empty: '',
	// combinations
	CmSp: ', ',
	ClSp: ': ',
	ScSp: '; ',
	// type
	undefined: 'undefined',
	number: 'number',
	string: 'string',
	boolean: 'boolean',
	Date_: 'Date',
	object: 'object',
	function_: 'function',
	// events
	click: 'click',
	dblclick: 'dblclick',
	change: 'change',
	changing_events: ['propertychange', 'keyup', 'input', 'paste', 'blur', 'change'],
	resize: 'resize',
	touchstart: 'touchstart',
	touchmove: 'touchmove',
	touchend: 'touchend',
	touchcancel: 'touchcancel',
	touchleave: 'touchleave',
	mousedown: 'mousedown',
	mousemove: 'mousemove',
	mousedrag: 'drag',
	mouseup: 'mouseup',
	mouseclick: 'click',
	// various
	display: 'display',
	width: 'width',
	height: 'height',
	block: 'block',
	none: 'none',
	inline: 'inline',
	inline_block: 'inline-block',
	selected: 'selected',
	panel_selected: 'panel_selected',
	playing: 'playing',
	checked: 'checked',
	focus: 'focus',
	blur: 'blur',
	visibility: 'visibility',
	visible: 'visible',
	hidden: 'hidden',
	// characters
	zero20: "00000000000000000000",
	num: "0123456789",
	hex: "0123456789abcdef",
	lower: "abcdefghijklmnopqrstuvwxyz",
	upper: "ABCDEFGHIJKLMNOPQRSTUVWXYZ",
	// html
	nbsp: '&nbsp;',
	br: '<br/>',
	//
	// FUNCTIONS
	// 
	// bare-bones event-handler delegate
	delegate: function ( _this, method ) {
		return function ( event ) {
			var args = Array.prototype.slice.call( arguments );
			return method.apply( _this, args );
		};
	}
};
// computed constants
CS.alpha = CS.lower + CS.upper;
CS.numAl = CS.num + CS.alpha;
CS.numLw = CS.num + CS.lower;
CS.numUp = CS.num + CS.upper;
//
//
var KEY = {
	enter: 13,
	shift: 16,
	ctrl: 17,
	alt: 18,
	pausebreak: 19,
	capslock: 20,
	escape: 27,
	pageup: 33,
	space: 32,
	pagedown: 34,
	end: 35,
	home: 36,
	arrowleft: 37,
	arrowup: 38,
	arrowright: 39,
	arrowdown: 40,
	insert: 45,
	delete_: 46,
	dot: 190
};
//
// typeof returns: "number", "string", "boolean", "object", "function", "undefined."
function def( o ) { return 'undefined' !== typeof o; }
function nll( o ) { return null === o || 'undefined' === typeof o; }
function cnt( o ) { return ( null !== o && 'object' === typeof o ) ? ( Object.keys ? Object.keys( o ).length : 0 ) : 0; }
function isNumeric( o ) { return null !== o && 'number' === typeof o; }
function isString( o ) { return null !== o && 'string' === typeof o; }
function isObject( o ) { return null !== o && 'object' === typeof o && !o.push; }
function isArray( o ) { return null !== o && 'object' === typeof o && !!o.push; }
function isFunction( o ) { return null !== o && 'function_' === typeof o && !!o.apply; }
function isBoolean( o ) { return null !== o && 'boolean' === typeof o; }
function isDate( o ) { return null !== o && 'Date' === typeof o; }
//
function empty( o ) {
	if ( nll( o ) ) return true;
	if ( !def( o.length ) ) return false;
	if ( 0 === o.length ) return true;
	return false;
}
//
// Wrapper for object methods.
function handler( object, method ) {
	return function () {
		return method.apply( object, arguments );
	};
}
//
// String utilities
//
if ( typeof String.prototype.startsWith !== 'function' ) {
	String.prototype.startsWith = function ( str ) {
		return this.slice( 0, str.length ) === str;
	};
}
if ( typeof String.prototype.endsWith !== 'function' ) {
	String.prototype.endsWith = function ( str ) {
		return this.slice( -str.length ) === str;
	};
}
if ( typeof String.prototype.xtrim !== 'function' ) {
	String.prototype.xtrim = function ( c ) {
		var l = new RegExp( '^' + c, 'g' );
		var r = new RegExp( c + '$', 'g' );
		return this.replace( l, '' ).replace( r, '' );
	};
}
if ( typeof String.prototype.format !== 'function' ) {
	String.prototype.format = function () {
		var args = arguments;
		return this.replace( /{(\d+)}/g, function ( match, number ) {
			return typeof args[number] !== 'undefined'
				? args[number]
				: match
				;
		} );
	};
}
// Left-trim a string, using whitespace for the optional chars regular expression.
function ltrim( str, chars ) {
	chars = chars || "\\s";
	return str.replace( new RegExp( "^[" + chars + "]+", "g" ), "" );
}
// Right-trim a string, using whitespace for the optional chars regular expression.
function rtrim( str, chars ) {
	chars = chars || "\\s";
	return str.replace( new RegExp( "[" + chars + "]+$", "g" ), "" );
}
// Trim a string, using whitespace for the optional chars regular expression.
function trim( str, chars ) {
	return ltrim( rtrim( str, chars ), chars );
}
// Replace all single or double quotes with Unicode Character 'right single quotation mark' (U+2019)
function noquotes( str ) {
	return str.replace( /['"]/g, "\u2019" );
}
// Put a string in single quotes. Does not escape existing quotes.
function q1( s ) { return "'" + s + "'"; }
// Put a string in double quotes. Does not escape existing quotes.
function q2( s ) { return '"' + s + '"'; }
// Format 'name="value"'.
function nv( n, v ) { return ' ' + n + '=' + '"' + v + '"'; }
//
function pad( num, size ) {
	if ( !num || !size || size < 2 || size > 20 ) return null;
	var s = CS.zero20 + num;
	return s.substr( s.length - size );
}
//
function capFirst( value ) {
	var len = value.length;
	var s1 = len >= 1 ? value.substr( 0, 1 ).toUpperCase() : '';
	var s2 = len >= 2 ? value.substr( 1 ) : '';
	return s1 + s2;
}
//
function currentDate() {
	var now = new Date();
	var s = pad( now.getFullYear(), 4 ) + '-';
	s += pad( 1 + now.getMonth(), 2 ) + '-';
	s += pad( now.getDate(), 2 );
	return s;
}
//
function currentTime() {
	var now = new Date();
	var s = pad( now.getHours(), 2 ) + ':';
	s += pad( now.getMinutes(), 2 ) + ':';
	s += pad( now.getSeconds(), 2 );
	return s;
}
//
//
// Float Utilities
//
// Test if the value is between min and max, including.
function between( min, val, max ) {
	return min <= val && val <= max;
}
// Test if a numeric value is equal to another value, for the given delta.
function equal( test, against, delta ) {
	return between( against - delta, test, against + delta );
}
// Make sure the 'val' is between min and max, inclusive.
function clamp( min, val, max ) {
	if ( val < min ) return min;
	if ( val > max ) return max;
	return val;
}
//
var rxFloat = new RegExp( '^[-]*(([0-9]+)|([.][0-9]+)|([0-9]+[.][0-9]*))$' );
//
// Convert a value into float. non-numeric input results in 0.0.
function uiFloat( value ) {
	if ( !def( value ) ) return;
	if ( null === value ) return null;
	if ( isString( value ) && !rxFloat.test( value ) ) return 0.0;
	return parseFloat( value );
}
//
// Convert a value into a percentage. non-numeric input results in 0.0.
function uiPercent( value ) {
	if ( !def( value ) ) return;
	if ( null === value ) return null;
	if ( isString( value ) && !rxFloat.test( value ) ) return 0.0;
	var result = parseFloat( value );
	return 100.0 * result;
}
// Convert a number into hexadecimal string
function dec2hex( n ) {
	if ( n < 0 ) {
		n = 0xFFFFFFFF + n + 1;
	}
	return n.toString( 16 ).toUpperCase();
}
//
// General Utilities
//
// Returns the number of milliseconds since midnight of January 1, 1970.
function tickCount() {
	if ( !Date.now ) return ( new Date() ).getTime();
	return Date.now();
}
// Returns the number of milliseconds in the current minute. Use this for debugging only!
function minuteTicks() {
	return tickCount() % 60000;
}
// NO-OP; function without functionality.
function noOp() { }
//
//
//
//
function randomInt( range ) {
	return Math.floor( Math.random() * range );
}
//
function randomChar() {
	return CS.numAl.substr( randomInt( CS.numAl.length ), 1 );
}
//
function randomID( size ) {
	var str = "";
	for ( var i = 0; i < size; i++ ) {
		str += randomChar();
	}
	return str;
}
//
function numUpChr( i ) {
	var s = '';
	var len = CS.numUp.length;
	do {
		var n = parseInt( i % len );
		var c = CS.numUp.substr( n, 1 );
		s = c + s;
		i = parseInt( i / len );
	} while ( i > 0 );
	return s;
}
//
// adapted from: http://joncom.be/code/javascript-rounding-errors/
function round( value, decimals ) {
	return Math.round( parseFloat( value ) * Math.pow( 10, decimals ) ) / Math.pow( 10, decimals );
}
//
function roundFixed( value, decimals ) {
	return Round( value, decimals ).toFixed( decimals );
}
//
//
function log2( n ) {
	var log2 = Math.log( 2 );
	var logN = Math.log( n );
	return logN / log2;
}
//
function escapeRegExp( string ) {
	var rx = new RegExp();
	rx.compile( "([.*+?^=!:${}()|\\[\\]\\/\\\\])", "g" );
	window.escapeRegExp = function ( string ) {
		return string.replace( rx, "\\$1" );
	};
	return escapeRegExp( string );
}
//
// Sort an array and remove all duplicate values
function sort_unique( arr ) {
	if ( arr.length === 0 ) return arr;
	arr = arr.sort();
	// store the first value into the returned array
	var ret = [arr[0]];
	// start loop at 1 as element 0 can never be a duplicate
	for ( var i = 1; i < arr.length; i++ ) {
		// push only unique values into the returned array
		if ( arr[i - 1] !== arr[i] ) {
			ret.push( arr[i] );
		}
	}
	return ret;
}
